#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, 
    std::vector<MyFace> , std::vector<MyEdge>  > {};

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.off>\n");
    return -1;
  }
  
  MyMesh m;

  if(vcg::tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=vcg::tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  vcg::tri::UpdateTopology<MyMesh>::FaceFace(m);
  
  int borderEdgeNum=0;
  for(int i=0;i<m.FN();++i)
    for(int j=0;j<3;++j)
    {
      if(m.face[i].FFp(j) == &(m.face[i]) )
        ++borderEdgeNum;
    }
  
  printf("Border Edge Number %i\n",borderEdgeNum);
  return 0;
}
